-- $Id$
local frame = CreateFrame("Frame", "DualSpeccerFrame")

local banner = "|cff00ff00DualSpeccer|r: "

local filtering = false
local filterRemaining = 0

local groupNames = {"primary", "secondary"}
local groupNamesCaps = {"Primary", "Secondary"}

local primarySpecSpellName = GetSpellInfo(63645)
local secondarySpecSpellName = GetSpellInfo(63644)

local specCache = {}

--[[
	TODO:
	- Some settings on controlling the text that appears in the Broker launcher.
	- Localization (AceLocale?), maybe get the spell learned messages from GlobalStrings
]]

--[[
	Information on zoning and talent info:
	
	On initial login, proper talent information isn't available until an onUpdate (as far as I've seen).
	On zoning to a different server, the following happens:
	- PLAYER_LEAVING_WORLD
	- Talent data becomes unavailable (returns 0/0/0)
	- PLAYER_TALENT_UPDATE
	- PLAYER_ENTERING_WORLD and related spam (BAG_UPDATE events etc)
	- Talent data becomes available
	- UPDATE_INSTANCE_INFO
	Teleporting around on the same continent doesn't interest us and nothing we care about fires.
]]

--[[
	What happens when switching specs:
	- UNIT_SPELLCAST_SENT player Activate Primary/Secondary Spec
	- UNIT_SPELLCAST_START player Activate Primary/Secondary Spec
	- CHAT_MSG_SYSTEM and SPELLS_CHANGED spam "You have unlearned blah"
	- CHAT_MSG_SYSTEM and LEARNED_SPELL_IN_TAB and SPELLS_CHANGED spam "You have learned blah"
	- UNIT_SPELLCAST_FAILED_QUIET player Activate Primary/Secondary Spec
	- UNIT_SPELLCAST_STOP player Activate Primary/Secondary Spec
	- PLAYER_TALENT_UPDATE
	- ACTIVE_TALENT_GROUP_CHANGED newGroupNumber, oldGroupNumber
	- ACTIONBAR_SLOT_CHANGED spam
	- UNIT_SPELLCAST_SUCCEEDED player Activate Primary/Secondary Spec
	- GLYPH_UPDATED/ADDED, SPELL_USABLE info and what not.
	
	Yes, we fail and then succeed at casting. I didn't make it...
]]

-- Localize some common functions.
local GetNumTalentGroups, GetActiveTalentGroup = GetNumTalentGroups, GetActiveTalentGroup

-- Updates the spec cache.
local function updateCache()
	for i = 1, GetNumTalentGroups() do
		specCache[i] = specCache[i] or {}
		local thisCache = specCache[i]
		TalentFrame_UpdateSpecInfoCache(thisCache, false, false, i)
		if thisCache.primaryTabIndex and thisCache.primaryTabIndex ~= 0 then
			thisCache.specName = thisCache[thisCache.primaryTabIndex].name
			thisCache.mainTabIcon = thisCache[thisCache.primaryTabIndex].icon
		else
			thisCache.specName = "Hybrid"
			thisCache.mainTabIcon = "Interface\\Icons\\Ability_Marksmanship"
		end
		thisCache.specGroupName = groupNamesCaps[i]
	end
end

-- Updates our Broker object.
local function update()
	local activeGroupNum = GetActiveTalentGroup()
	local curCache = specCache[activeGroupNum]
	
	DualSpeccerLDB.text = curCache.specGroupName
	DualSpeccerLDB.icon = curCache.mainTabIcon
end

local function showTooltip(tooltip)
	local curCache = specCache[GetActiveTalentGroup()]
	
	tooltip:AddLine("DualSpeccer")
	tooltip:AddDoubleLine("Active spec", ("%s (%s)"):format(curCache.specGroupName, curCache.specName))
	tooltip:AddDoubleLine("Primary spec", ("%s (%d/%d/%d)"):format(specCache[1].specName, specCache[1][1].pointsSpent, specCache[1][2].pointsSpent, specCache[1][3].pointsSpent))
	if GetNumTalentGroups() >= 2 then
		tooltip:AddDoubleLine("Secondary spec", ("%s (%d/%d/%d)"):format(specCache[2].specName, specCache[2][1].pointsSpent, specCache[2][2].pointsSpent, specCache[2][3].pointsSpent))
		tooltip:AddLine(" ")
		tooltip:AddLine("|cff00ff00Click|r to switch to your other specialization.")
	end
end

local function switchSpecs()
	if GetNumTalentGroups() < 2 then
		DEFAULT_CHAT_FRAME:AddMessage(banner .. "This character does not have a second specialization.")
		return
	end
	
	local curSpec = GetActiveTalentGroup()
	-- -1 + 3 = 2 and -2 + 3 = 1.
	local newSpec = -curSpec + 3
	SetActiveTalentGroup(newSpec)
end

-- Prints a nice message telling users that they switched specs.
local function congrats(newGroup)
	DEFAULT_CHAT_FRAME:AddMessage(banner .. "Switched to " .. groupNames[newGroup] .. " talent spec.")
end

-- Our chat filter function.
local function filter(self, event, msg)
	-- You have unlearned Blahblah.
	-- You have learned a new spell: Blahblah.
	-- You have learned a new ability: Blahblah.
	if filtering and (msg:find("You have unlearned") ~= nil or msg:find("You have learned a new") ~= nil) then
		return true
	end
end

-- Stops filtering chat.
local function stopFilter()
	if not filtering then return end
	filtering = false
	filterRemaining = 0
	frame:Hide()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
end

-- OnUpdate function.
local function filterOnUpdate(self, elapsed)
	filterRemaining = filterRemaining - elapsed
	if filterRemaining <= 0 then
		stopFilter()
	end
end

-- Starts filtering chat.
local function startFilter()
	if filtering then return end
	filtering = true
	filterRemaining = 10
	frame:SetScript("OnUpdate", filterOnUpdate)
	frame:Show()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
end

-- OnEvent function. Handles UNIT_SPELLCAST_SENT/INTERRUPTED and ACTIVE_TALENT_GROUP_CHANGED.
local function onEvent(self, event, ...)
	if event == "UNIT_SPELLCAST_START" then
		local unit, spellName = ...
		if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
			-- We are now switching specs. Hide learned messages for the next ten seconds.
			startFilter()
		end
	elseif event == "UNIT_SPELLCAST_INTERRUPTED" then
		local unit, spellName = ...
		if unit == "player" and (spellName == primarySpecSpellName or spellName == secondarySpecSpellName) then
			-- We moved (or something). End filtering.
			stopFilter()
		end
	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		-- This event fires after all of the system crap (the messages get pushed before anything else, including spell success).
		local newGroup, oldGroup = ...
		stopFilter()
		congrats(newGroup)
		update()
	elseif event == "PLAYER_TALENT_UPDATE" then
		-- P_T_U fires when the player puts a talent point or just completed a spec swap.
		updateCache()
		update()
	elseif event == "UPDATE_INSTANCE_INFO" then
		-- See the information near the top of this file for the reasoning behind using this event.
		updateCache()
		update()
	end
end

-- This OnUpdate function is only used for the first frame. Talent data isn't availabe until sometime after P_E_W.
-- To avoid any potential issues, we only do the majority of our work after this has happened.
local function onFirstUpdate(self, elapsed)
	updateCache()
	update()
	self:Hide()
	self:SetScript("OnUpdate", nil)
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	self:RegisterEvent("UPDATE_INSTANCE_INFO")
end

frame:SetScript("OnEvent", onEvent)
frame:SetScript("OnUpdate", onFirstUpdate)
frame:Show()

DualSpeccerLDB = LibStub("LibDataBroker-1.1"):NewDataObject("DualSpeccer", {
	type = "launcher",
	text = "DualSpeccer",
	icon = "Interface\\Icons\\Ability_Marksmanship",
	tocname = "DualSpeccer",
	OnClick = switchSpecs,
	OnTooltipShow = showTooltip
})
